package be.iminds.ilabt.jfed.rspec.model.imutable_impl;

import be.iminds.ilabt.jfed.rspec.basic_model.BasicStringRspec;
import be.iminds.ilabt.jfed.rspec.model.RspecInterface;
import be.iminds.ilabt.jfed.rspec.model.RspecLink;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ImmutableRspecInterface implements RspecInterface {
    private static final Logger LOG = LoggerFactory.getLogger(ImmutableRspecInterface.class);

    protected final ImmutableModelRspec modelRspec;

    protected final String clientId;
    protected final GeniUrn sliverId;
    protected final String componentId;
    protected final String macAddress;
    protected final Integer felixVlan;

    protected final String componentName;
    protected final String role;
    protected final String publicIpv4;

    protected final List<ExtraXml> extraXml;
    protected final List<ExtraXml> extraXmlForRef;
    protected final List<IpAddress> ipAddresses;

    protected boolean forceFinal; //set to true when copy has been completed. This causes node and link to be unmodifyable.
    @Nullable protected ImmutableRspecNode node; //not final to allow deep copy...
    @Nullable protected ImmutableRspecLink link; //not final to allow deep copy...

    ImmutableRspecInterface(ImmutableModelRspec modelRspec, @Nullable ImmutableRspecNode node, @Nullable ImmutableRspecLink link, RspecInterface orig) {
        assert node == null || node.getRspec() == modelRspec;
        assert link == null || link.getRspec() == modelRspec;

        this.modelRspec = modelRspec;

        this.clientId = orig.getClientId();
        this.sliverId = orig.getSliverId();
        this.componentId = orig.getComponentId();
        this.componentName = orig.getComponentName();
        this.role = orig.getRole();
        this.publicIpv4 = orig.getPublicIpv4();
        this.macAddress = orig.getMacAddress();
        this.felixVlan = orig.getFelixVlan();
        this.ipAddresses = Collections.unmodifiableList(new ArrayList<>(orig.getIpAddresses()));
        this.extraXml = Collections.unmodifiableList(new ArrayList<>(orig.getExtraXml()));
        this.extraXmlForRef = Collections.unmodifiableList(new ArrayList<>(orig.getRefExtraXml()));

        if (node != null) {
            bindNode(node);
        }
        if (link != null) {
            bindLink(link);
        }
    }

    @Override
    public void overwritePropertiesWith(RspecInterface other, boolean overwriteWithNull) {
        throw new UnsupportedOperationException();
    }

    public void setForceFinal(boolean forceFinal) {
        this.forceFinal = forceFinal;
    }

    @Override
    public void bindLink(RspecLink rspecLink) {
        if (forceFinal) throw new UnsupportedOperationException();

        assert rspecLink instanceof ImmutableRspecLink;
        assert this.link == null;

        this.link = (ImmutableRspecLink) rspecLink;

        if (this.node != null && this.link != null) {
            //iface is now fully bound
            this.forceFinal = true;
        }
    }

    @Override
    public void bindNode(RspecNode rspecNode) {
        if (forceFinal) throw new UnsupportedOperationException();

        assert rspecNode instanceof ImmutableRspecNode;
        assert this.node == null;

        this.node = (ImmutableRspecNode) rspecNode;

        if (this.node != null && this.link != null) {
            //iface is now fully bound
            this.forceFinal = true;
        }
    }

    @Override
    public void moveToNode(RspecNode newRspecNode) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void moveToLink(RspecLink newRspecLink) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isBound() {
        return node != null && link != null;
    }

    @Override
    public boolean isUnbound() {
        return node == null || link == null;
    }

    @Override
    public boolean isLinkUnbound() {
        return link == null;
    }

    @Override
    public boolean isNodeUnbound() {
        return node == null;
    }

    @Override
    public boolean isLinkBound() {
        return link != null;
    }

    @Override
    public boolean isNodeBound() {
        return node != null;
    }

    @Override
    public String getUniqueId() {
        return BasicStringRspec.makeUniqueInterfaceId(
                clientId,
                componentId,
                node == null ? null : node.getUniqueId()
        );
    }

    @Override
    public String getClientId() {
        return clientId;
    }

    @Override
    public void setClientId(String clientId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getComponentId() {
        return componentId;
    }

    @Override
    public void setComponentId(String componentId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getPublicIpv4() {
        return publicIpv4;
    }

    public void setPublicIpv4(String publicIpv4) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getRole() {
        return role;
    }

    public void setRole(String role) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getComponentName() {
        return componentName;
    }

    public void setComponentName(String componentName) {
        throw new UnsupportedOperationException();
    }

    @Override
    public GeniUrn getSliverId() {
        return sliverId;
    }

    @Override
    public void setSliverId(String sliverId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setSliverId(GeniUrn sliverId) {
        throw new UnsupportedOperationException();

    }

    @Override
    public String getMacAddress() {
        return macAddress;
    }

    @Override
    public void setMacAddress(String value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Integer getFelixVlan() {
        return felixVlan;
    }

    @Override
    public void setFelixVlan(Integer felixVlan) {
        throw new UnsupportedOperationException();
    }

    @Override
    public List<IpAddress> getIpAddresses() {
        return ipAddresses;
    }

    @Override
    public List<ExtraXml> getExtraXml() {
        return extraXml;
    }

    @Override
    public List<ExtraXml> getRefExtraXml() {
        return extraXmlForRef;
    }

    @Override
    public ImmutableRspecNode getNode() {
        if (this.node == null)
            throw new IllegalStateException("bindNode was not yet called");
        return node;
    }

    @Override
    public ImmutableRspecLink getLink() {
        if (this.link == null)
            throw new IllegalStateException("bindLink was not yet called");
        return link;
    }

    @Override
    public boolean delete() {
        throw new UnsupportedOperationException();
    }

    @Override
    public ImmutableModelRspec getRspec() {
        return modelRspec;
    }

    @Override
    public String toString() {
        return "RspecInterface{id=" + clientId + '}';
    }
}
